% tetris.m - a set cover problem inspired by Tetris
%
% See the bottom of the file for runtime benchmark data

mss_config('default_solver', 'blt');
% mss_config('default_solver', 'yices');

msg = 'Starting...'
% positions on an m by n rectangular grid
m = 3;  % nrows
n = 3;  % ncols
% x(i,j) represents the choice of using a peice whose
% upper left most part covers position (i,j)
x = symbolic('x', m+1, n+2, 'integer');

% objective function
sym_obj = sum(sum(x));

% {0,1} constraints on variables
p = true;
for i = 1:(m+1)
    for j = 1:(n+2)
        p = p && 0 <= x(i,j) && x(i,j) <= 1;
    end
end

% covering constraints for a tetris like peice
%
%   **
%  **
%
for i = 1:m
    for j = 1:n
        q = x(i,j) + x(i,j+1) + x(i+1,j+1) + x(i+1,j+2);
        p = p && 1 <= q && q <= 4;
    end
end

t0 = clock;
sat = true
soln = []
obj_c = true  % extra constraint added in the loop
while sat

    [r,c] = check_sat(obj_c && p);
    if r == 1
        soln = c.x;
        obj_v = sum(sum(soln));
        obj_c = 1 <= sym_obj && sym_obj <= (obj_v - 1);
        msg = 'Found feasible solution'
        obj_v
        t1 = clock;
        msg = 'Time ellapsed:'
        time = etime(t1, t0)
    else
        msg = 'No better feasible solutions exist'
        sat = false;
    end
end
t1 = clock;

% msg = 'Verifying solution...'
% soln
% for i = 1:m
%     for j = 1:n
%         cov(i,j) = soln(i,j) + soln(i,j+1) + soln(i+1,j+1) + soln(i+1,j+2);
%     end
% end
% msg = 'Coverage:'
% cov
msg = 'Time ellapsed:'
time = etime(t1, t0)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% benchmarks as of 2014-07-17
%
% fields: dim timeSoln timeProof
% * = 1 hr timeout
%
% BLT
%
% 20 0.019 17.0
% 24 0.036 56.3
% 30 0.041 138
% 35 0.061 156
% 42 0.046 1910
% 48 0.220 *
% 56 0.222 *
% 64 28.00 *
%
% Yices2
%
% 20 0.170 0.178
% 24 0.200 0.208
% 30 0.261 0.271
% 35 0.366 0.374
% 42 0.449 0.460
% 48 0.600 0.613
% 56 0.721 0.734
% 64 12.80 12.90
