function TestHarness()
    [cc s] = mkTestProblem();
    [nr nr] = size(cc);

    start = clock;

    b      = symbolic('b',  1, nr, 'logical');

    % Slack space, making all constraints feasible, but providing a measure of infeasiblity.
    totalTheoryTime  = 0;

    learnings        = [];
    unsatSATconflict = 2; % conflict
    iterations       = 0;
    while unsatSATconflict == 2
        [unsatSATconflict assignments theoryTime] = CombinedSolver_inner(b,cc,learnings,s);
        totalTheoryTime = totalTheoryTime + theoryTime;
        if unsatSATconflict == 2
            learnings = assignments;
        end
        iterations = iterations + 1;
    end

    if unsatSATconflict == 0
        'unsat'
    elseif unsatSATconflict == 1
        'sat'
        x = assignments.x
        y = assignments.y
        attackVector = assignments.b
    else
        'conflict?'
        unsatSATconflict
        'Impossible: A new conflict was learned, it seems, and the while loop terminated...'
    end
    totalIterations = iterations

    'Time consumed by the theory solver (convex opt): '
    theorySolverTime = totalTheoryTime
    'Out of a total time of: '
    runtime = etime(clock,start)
end

% Construct our test problem
%    cc   :  The constraints on our optimization problem (2d space with intersecting circles)
%    s    :  The sat parameter indicating how many constraints must be satisfied (min sat)
function [cc s] = mkTestProblem()
    x  = symbolic('x', 'real');
    y  = symbolic('y', 'real');
    s  = 5;  % Set s < 5 to get unsat
    % Convex constraints are implicity <= 0.
    problem1  = x - 5;                         % region x < 5
    problem2  = ((x-10)^2 + (y+0.5)^2 - 1);    % circle radius 1 centered at y=10
    problem3  = ((x-1)^2/16 + (y+0.5)^2 - 1);  % elipse (1,-0.5)
    problem4  = 100 - y;                       % region y > 100
    problem5  = (x)^2 - 10;                    % Parabola based y=-10
    problem6  = ((x)^2 + (y+10)^2 - 1);        % circle radius 1 centered at y=-10
    problem7  = ((x)^2 + (y-10)^2 - 1);        % circle radius 1 centered at y=10
    problem8  = ((x-4.5)^2 + y^2 - 1);         % circle radius 1 centered at x=4.5
    problem9  = ((x-5)^2 + y^2 - 1);           % circle radius 1 centered at x=5
    problem10 = ((x-5.5)^2 + y^2 - 4);         % circle radius 2 centered at x=5.5
    cc = [ problem1 problem2 problem3 problem4 problem5 problem6 problem7 problem8 problem9 problem10 ];
end
