% This function encodes the game of life by counting the number of set bits
% in each 3x1 column, and then sums up to check whether each 3x3 square has
% 3 or 4 bits set.
function y = conway3(x)
[r,c] = size(x);

% Shift x indices down
x_d = shift_down(x, false(1, c));
% Shift x indices up
x_u = shift_up(false(1, c), x);

row0_c = ~x_d & ~x & ~x_u;
row1_c = (x_d & ~x & ~x_u) | (~x_d &  x & ~x_u) | (~x_d & ~x &  x_u);
row2_c = (x_d &  x & ~x_u) | ( x_d & ~x &  x_u) | (~x_d &  x &  x_u);
row3_c =  x_d &  x &  x_u;

row0_l = shift_left(row0_c,  true(r,1));
row1_l = shift_left(row1_c, false(r,1));
row2_l = shift_left(row2_c, false(r,1));
row3_l = shift_left(row3_c, false(r,1));

row0_r = shift_right( true(r,1), row0_c);
row1_r = shift_right(false(r,1), row1_c);
row2_r = shift_right(false(r,1), row2_c);
row3_r = shift_right(false(r,1), row3_c);

set3 = row0_l & row0_c & row3_r ...
     | row1_l & row0_c & row2_r ...
     | row2_l & row0_c & row1_r ...
     | row3_l & row0_c & row0_r ...
     | row0_l & row1_c & row2_r ...
     | row1_l & row1_c & row1_r ...
     | row2_l & row1_c & row0_r ...
     | row0_l & row2_c & row1_r ...
     | row1_l & row2_c & row0_r ...
     | row0_l & row3_c & row0_r;


set4 = row0_l & row1_c & row3_r ...
     | row1_l & row1_c & row2_r ...
     | row2_l & row1_c & row1_r ...
     | row3_l & row1_c & row0_r ...
     | row0_l & row2_c & row2_r ...
     | row1_l & row2_c & row1_r ...
     | row2_l & row2_c & row0_r ...
     | row0_l & row3_c & row1_r ...
     | row1_l & row3_c & row0_r;

y = set3 | x & set4;



%{
We have an alternate encoding that checks whether
3 or 4 variables are set.

This encoding unfortunately is slower.

row01_c = ~x & ~x_u | ~x_d & ~x_u | ~x_d & ~x;
row12_c = x_d & ~x_u ...
        | x_d & ~x ...
        | ~x_d & x ...
        | ~x_d & x_u ...
        | x & ~x_u ...
        | ~x & x_u;
row23_c = x & x_u | x_d & x_u | x_d & x;

row01_l = shift_left(row01_c,  true(r,1));
row12_l = shift_left(row12_c, false(r,1));
row23_l = shift_left(row23_c, false(r,1));

row01_r = shift_right( true(r,1), row01_c);
row12_r = shift_right(false(r,1), row12_c);
row23_r = shift_right(false(r,1), row23_c);


se34 = row0_l & row1_c & row23_r ...
     | row0_l & row2_c & row12_r ...
     | row0_l & row3_c & row01_r ...
     | row1_l & row0_c & row23_r ...
     | row1_l & row1_c & row12_r ...
     | row1_l & row2_c & row01_r ...
     | row2_l & row0_c & row12_r ...
     | row2_l & row1_c & row01_r ...
     | row3_l & row0_c & row01_r ...
        ...
     | row0_l & row1_r & row23_c ...
     | row0_l & row2_r & row12_c ...
     | row0_l & row3_r & row01_c ...
     | row1_l & row0_r & row23_c ...
     | row1_l & row1_r & row12_c ...
     | row1_l & row2_r & row01_c ...
     | row2_l & row0_r & row12_c ...
     | row2_l & row1_r & row01_c ...
     | row3_l & row0_r & row01_c ...
         ...
     | row0_c & row1_r & row23_l ...
     | row0_c & row2_r & row12_l ...
     | row0_c & row3_r & row01_l ...
     | row1_c & row0_r & row23_l ...
     | row1_c & row1_r & row12_l ...
     | row1_c & row2_r & row01_l ...
     | row2_c & row0_r & row12_l ...
     | row2_c & row1_r & row01_l ...
     | row3_c & row0_r & row01_l;

y = set3 | x & se34;
%}


end

function y = shift_down(x,r)
y = [ x(2:end,:) ; r ];
end

function y = shift_up(r, x)
y = [ r ; x(1:end-1,:) ];
end

function y = shift_left(x,c)
y = [ x(:,2:end) c ];
end

function y = shift_right(c,x)
y = [ c x(:,1:end-1) ];
end
