function [y,cond] = conway_cnf(x)
y = x;

[rows,cols] = size(x);

cond = true;

for i = 1:rows
  for j = 1:cols
    a = [ i > 1    && j > 1    && x(i-1,j-1)
          i > 1    &&             x(i-1,j)
          i > 1    && j < cols && x(i-1,j+1)
                      j > 1    && x(i,j-1)
                      j < cols && x(i,j+1)
          i < rows && j > 1    && x(i+1,j-1)
          i < rows &&             x(i+1,j)
          i < rows && j < cols && x(i+1,j+1)
        ];
    a = a';

    cur = x(i,j);
    next = symbolic('', 'logical');

    % Add 8 clauses for one variable set.
    for i1 = 1:8
      set_idx = arrayfun(@(i) i == i1, 1:8);
      cond = cond & (any(~set_idx & a) | ~next);
    end

    % Add 56 clauses for two or fewer variables set and cur unset.
    for i1 = 1:7
      for i2 = i1+1:8
        % Get mask that sets positions at i1 or i2 to 1.
        set_idx = arrayfun(@(i) any(i == [i1 i2]), 1:8);
        % Require that all indices but i1 or i2 to be unset implies next is unset.
        cond = cond & implies(~cur & all(set_idx | ~a), ~next);
      end
    end

    % Add clauses for two or variables set.
    for dc1 = 1:8
      for i1 = 1:7
        if i1 == dc1
          continue
        end
        for i2 = i1+1:8
          if i2 == dc1 continue; end
          indices = [i1 i2];

          % Compute clause that asserts i1 and i2 are set, and indices
          % other than dc1 are unset.  dc1 is a don't care bit and may
          % be set or not.
          c = true;
          for k=1:8
            if k == dc1 continue; end
            c = c & any(k == indices) == a(k);
          end

          cond = cond & implies(cur & c, next);
        end
      end
    end

    % Add clauses for 3-variables set.
    for i1 = 1:6
      for i2 = i1+1:7
        for i3 = i2+1:8
          % Add clause asserting that i1 i2 and i3 are set
          % and other variables are unset implies next is set.
          indices = [i1 i2 i3];
          set_idx = arrayfun(@(i) any(i == indices), 1:8);
          cond = cond & implies(all(set_idx == a), next);
        end
      end
    end

    % Add clauses for asserting 4-variables set implies false.
    for i1 = 1:5
      for i2 = i1+1:6
        for i3 = i2+1:7
          for i4 = i3+1:8
            cond = cond & implies(a(i1) & a(i2) & a(i3) & a(i4), ~next);
          end
        end
      end
    end

    y(i,j) = next;
  end

end

end

function r = implies(a,b)
    r = ~a | b;
end