% sym_area.m
%
% This example demonstrates an object which depends on a symbolic value
% 'p'. For the simulator to work, both branches must result in 'o' being
% of the same class.
%
% For the non-linear problems below, you must have yices with mcsat enabled
% or use Z3.

mss_config('default_solver', 'z3');

p = symbolic('p', 1, 'logical');

% construct different Triangle objects depending on 'p'
o = choose_wisely(p);

% A non-linear, UNSAT problem
cond = o.area() == 10;
[r,c] = check_sat(cond)

% Another non-linear, SAT problem
cond = o.perimeter_lower_bound() < 20;
[r,c] = check_sat(cond)

% A linear, SAT problem
o.w = 3;
cond = o.area() == 15;
[r,c] = check_sat(cond)
