% This is the basic "incrementally greedy" search procedure
% for the 1D string-packing problem.

% Parameters to the problem
len        = 1000; % how long is the input stream
width      =    5; % how wide is the narrow sliding avg. window
extra      =   30; % how much wider is the wide sliding avg. window
maxsearch  =   50; % how far forward to search for a location before resetting
limit      =    1; % what is the limit for the wide sliding average?
searchinc  =    5; % how many spaces to skip when we search foraward
inputlimit =  300; % maximum distance from 0 an input value can be

% the strings we want to pack
segments  = { 'hello!asdf' ...
            , 'whatup'  ...
            , 'yall' ...
            , 'howdy', 'pardner' ...
            , 'WHATDO', 'YOUWANT' ...
            , 'howareyou' ...
            , 'my man'
            };
numsegs   = size(segments, 2);

% The symbolic input vector we are searching for
input     = symbolic_array('input', 1, len, 'integer');

% The vector of locations at which we place the strings
locs      = symbolic('locs', 1, numsegs+1, 'integer');

% symbolically calculate the narrow sliding window avg.
y = slide_sym( input, width, 1, 'y', 'real' );
y_len = size(y, 2);

% symbolically calculate the wide sliding window avg.
z = slide_sym( input, width+extra, 1, 'z', 'real' );

minsep = 1;

locs_ok = locs(1) >= 50 & locs(numsegs) <= y_len - 50;
for i = 1:numsegs
    % every location is in range
    locs_ok = locs_ok & minsep <= locs(i) & ...
              locs(i) + size(segments{i},2) + minsep < y_len;

    for j = 1:numsegs
        if i < j
            ij_disjoint = locs(i) + size(segments{i},2) + minsep <= locs(j);
            % sym_locs(j) + size(segments{j},2) + minsep <= sym_locs(i);
            locs_ok = locs_ok & ij_disjoint;
        end
    end
end

p = locs_ok;
for j = 1:numsegs
    segsize = size(segments{j}, 2);
    for k = 1:segsize
        p = p & (y(locs(j) + k - 1) == segments{j}(k));
    end
    if( j < numsegs )
        p = p & (y( locs(j) + segsize ) == (locs(j+1) - (locs(j) + segsize)));
    else
        p = p & (y( locs(j) + segsize ) == 0 );
    end
end

% Here we bound the size of the larger sliding average:
% this has the general effect of forcing the input
% vector to be roughly centered around 0.
p = p & all ( z <= limit ) & all ( z >= -limit );

% We also directly bound the magnitude of the input values
%p = p & all ( input <= inputlimit ) & all (input >= -inputlimit );

write_yices( 'string_pack_naive.yices', p );
[c r] = check_sat( p )


% Print the results of the narrow sliding average
q = slide( r.input, width, 1 )

% Print the results of the wide sliding average
w = slide( r.input, width+extra, 1 )

% Print the segments of the output where we expect to find
% the desired strings packed
for i = 1:numsegs
  q( locs(i) : (locs(i)+size(segments{i},2)) )
end
